/*
 * Author: Jake G 
 * Date: 2024-09-01 
 * filename: test_Relays.c
 * description: module_purpose
 */

#include "CppUTest/CommandLineTestRunner.h"
#include "CppUTestExt/MockSupport.h"

extern "C" 
{
#include "Relays.h"
#include "TimerMock.h"
}

void zero_relay_struct(Relay *relay)
{
    relay->input_pin = 0;
    relay->output_pin = 0;
    relay->break_time = 0;
    relay->make_time = 0;
    relay->disabled_fpc = false;
    *(uint8_t *)relay->output_port = 0x00;
    *(uint8_t *)relay->input_port = 0x00;
}

TEST_GROUP(test_Relays)
{
    uint8_t fake_input_port;
    uint8_t fake_ouput_port;
    Relay relay;

    void setup()
    {
        relay.input_port = &fake_input_port;
        relay.output_port = &fake_ouput_port;
        zero_relay_struct(&relay);
    }
    void teardown()
    {
        fake_input_port = 0x00;
        fake_ouput_port = 0x00;
    }
};


TEST(test_Relays, CreateRelayStruct)
{
    CHECK(relay.input_port == &fake_input_port);
}

TEST(test_Relays, DisabledForPowerCycleSetsDisabledBit)
{
    Relay_DisableForPowerCycle(&relay);
    CHECK_TRUE(relay.disabled_fpc);
}

TEST(test_Relays, EnableSetsCorrectPinsInPort)
{
    for(uint8_t i = 0; i <= 7; i++) {
        zero_relay_struct(&relay);
        relay.output_pin = i;
        Relay_Enable(&relay);
        CHECK_EQUAL((1<<i), *(uint8_t *)relay.output_port);
    }
}

TEST(test_Relays, EnableDoesNothingOnPCDisabledRelay)
{
    Relay_DisableForPowerCycle(&relay);
    Relay_Enable(&relay);
    CHECK_EQUAL(0x00, *(uint8_t *)relay.output_port);
}


TEST(test_Relays, DisableClearsCorrectPinsInPort)
{
    for(uint8_t i = 0; i <= 7; i++) {
        zero_relay_struct(&relay);
        *(uint8_t *)relay.output_port = 0xFF;
        relay.output_pin = i;
        Relay_Disable(&relay);
        CHECK_EQUAL(0xFF & ~(1<<i), *(uint8_t *)relay.output_port);
    }
}

TEST(test_Relays, DisableDoesNothingOnPCDisabledRelay)
{
    Relay_DisableForPowerCycle(&relay);
    *(uint8_t *)relay.output_port = 0xFF;

    Relay_Disable(&relay);
    CHECK_EQUAL(0xFF, *(uint8_t *)relay.output_port);
}

TEST(test_Relays, ReadStateOfUinitializedReadsFalse)
{
    for(uint8_t i = 0; i <= 7; i++){
        zero_relay_struct(&relay);
        relay.input_pin = i;
        CHECK_FALSE(Relay_ReadState(&relay));
    }
}

TEST(test_Relays, ReadStateOfEnabledRelayReadsTrue)
{

    for(uint8_t i = 0; i <= 7; i++){
        zero_relay_struct(&relay);
        relay.input_pin = i;
        *(uint8_t *)relay.input_port |= (1<<i);
        CHECK_TRUE(Relay_ReadState(&relay));
    }

    //Ensures that it's accounting for bit number.
    for(uint8_t i = 0; i <= 7; i++){
        zero_relay_struct(&relay);
        relay.input_pin = 7 - i;
        *(uint8_t *)relay.input_port |= (1<<i);
        CHECK_FALSE(Relay_ReadState(&relay));
    }
}


TEST_GROUP(test_RelaysTimer)
{
    uint8_t fake_input_port;
    uint8_t fake_ouput_port;
    Relay relay;

    void setup() {
        relay.input_port = &fake_input_port;
        relay.output_port = &fake_ouput_port;
        zero_relay_struct(&relay);   
    }

    void teardown() {
        fake_input_port = 0x00;
        fake_ouput_port = 0x00;
        mock().clear();
    }
};

TEST(test_RelaysTimer, MakeTimeCallsCorrectly)
{
    uint16_t fake_return = 56;
    mock().expectOneCall("Timer_Start");
    mock().expectOneCall("Timer_Stop");
    mock().expectOneCall("Timer_GetOverflowCount")
        .andReturnValue(fake_return);

    //Set the input hi
    *(uint8_t *)relay.input_port |= (1 << relay.input_pin);

    Relay_MeasureMakeTime(&relay);
    CHECK_EQUAL(fake_return, relay.make_time);
    CHECK_EQUAL(0x01, fake_ouput_port);

    mock().checkExpectations();
}


TEST(test_RelaysTimer, BreakTimeCallsCorrectly)
{
    //We should start by setting the bit for it high in the output reg.
    fake_ouput_port |= (1<<relay.output_pin);
    uint16_t fake_return = 56;
    mock().expectOneCall("Timer_Start");
    mock().expectOneCall("Timer_Stop");
    mock().expectOneCall("Timer_GetOverflowCount")
        .andReturnValue(fake_return);

    Relay_MeasureBreakTime(&relay);
    CHECK_EQUAL(fake_return, relay.break_time);
    CHECK_EQUAL(0x00, fake_ouput_port);

    mock().checkExpectations();
}
